
package org.ow2.petals.bc.jms.listener;

import java.util.logging.Level;

import javax.jms.JMSException;
import javax.jms.MessageConsumer;

import org.ow2.petals.bc.jms.connection.JMSConsumerConnection;
import org.ow2.petals.bc.jms.connection.JMSConsumerConnectionFactory;
import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.exception.TechnicalException;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;
import org.ow2.petals.component.framework.su.ServiceUnitManager;

public class JMSExternalListener extends AbstractExternalListener {

    private JMSConsumerConnectionFactory connectionFactory;

    private JMSConsumerConnection conn;

    @Override
    protected void init() {
        try {
            createConnectionFactory();
        } catch (TechnicalException e) {
            getLogger().log(Level.SEVERE, "Can't create consumer connection factory", e);
        }
    }

    private void createConnectionFactory() throws TechnicalException {
        String suRootPath = ((ServiceUnitManager) (getComponent().getServiceUnitManager()))
                .getSuDataHandlerForConsumeProvide(getConsumes()).getInstallRoot();

        this.connectionFactory = new JMSConsumerConnectionFactory(getExtensions(), suRootPath,
                component.getLogger());
    }

    @Override
    protected void start() throws PEtALSCDKException {
        try {
            this.conn = (JMSConsumerConnection) connectionFactory.makeObject();
            connectionFactory.activateObject(conn);
        } catch (Exception e1) {
            getLogger().log(Level.SEVERE,
                    "Can not retrieve a JMS connection for consumes : " + getConsumes(), e1);
        }

        if (conn != null) {
            JMSMessageListener jmsMessageListener = new JMSMessageListener(this);
            MessageConsumer consumer = conn.getMessageConsumer();
            try {
                if (consumer != null) {
                    consumer.setMessageListener(jmsMessageListener);
                }
            } catch (JMSException e) {
                throw new PEtALSCDKException(
                        "Can not set the jms messageListener for the jms consumer.", e);
            }
        }

    }

    @Override
    protected void stop() throws PEtALSCDKException {
        try {
            if (this.conn != null) {
                connectionFactory.passivateObject(this.conn);
                connectionFactory.destroyObject(this.conn);
            }
        } catch (Exception e) {
            throw new PEtALSCDKException("Can not destroy a JMS connection for consumes : "
                    + getConsumes(), e);
        }
    }

}
