
package org.ow2.petals.bc.jms.su;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.pool.impl.GenericObjectPool;
import org.ow2.petals.bc.jms.JmsBC;
import org.ow2.petals.bc.jms.connection.JMSProviderConnectionFactory;
import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.exception.TechnicalException;
import org.ow2.petals.component.framework.su.ServiceUnitListener;
import org.ow2.petals.component.framework.util.Extensions;
import org.ow2.petals.jbi.descriptor.JBIDescriptor;
import org.ow2.petals.jbi.descriptor.Provides;
import org.ow2.petals.jbi.descriptor.Services;

public class JmsServiceUnitListener implements ServiceUnitListener {

    public final static String MAX_ACTIVE = "max-active";

    public final static String MAX_IDLE = "max-idle";

    public final static String MAX_WAIT = "max-wait";

    public final static String TIME_BETWEEN_EVICTION_RUNS_MILLIS = "time-between-eviction-runs-millis";

    public final static String MIN_EVICTABLE_IDLE_TIME_MILLIS = "min-evictable-idle-time-millis";

    public final static String TEST_WHILE_IDLE = "test-while-idle";

    private JmsBC component;

    public JmsServiceUnitListener(JmsBC component) {
        super();
        this.component = component;
    }

    public void onSUDeployed(String serviceUnitName, String suRootPath, JBIDescriptor descriptor)
            throws PEtALSCDKException {
        for (Provides provides : getProvidesList(descriptor)) {
            try {
                registerConnectionPool(provides, suRootPath);
            } catch (TechnicalException e) {
                throw new PEtALSCDKException("Error creating a connection pool for SU: "
                        + suRootPath, e);
            }
        }
    }

    public void onSUStarted(String serviceUnitName) throws PEtALSCDKException {
        // TODO Create some connections here

    }

    public void onSUStopped(String serviceUnitName) throws PEtALSCDKException {
        // TODO remove all connections from the pool ?

    }

    public void onSUUndeployed(String serviceUnitName, String suRootPath, JBIDescriptor descriptor)
            throws PEtALSCDKException {
        for (Provides provides : getProvidesList(descriptor)) {
            try {
                unregisterConnectionPool(provides);
            } catch (TechnicalException e) {
                throw new PEtALSCDKException(e.getMessage(), e);
            }
        }
    }

    private List<Provides> getProvidesList(JBIDescriptor descriptor) {
        List<Provides> providesList = new ArrayList<Provides>();
        Services services = descriptor.getServices();
        if (services != null) {
            providesList = services.getProvides();
        }
        return providesList;
    }

    private void registerConnectionPool(Provides provides, String suRootPath)
            throws TechnicalException, PEtALSCDKException {
        Extensions extensions = null;
        if (provides.getExtensions() != null) {
            extensions = component.getExtensions(provides);
        }
        if (extensions != null) {
            int maxActive = Integer.parseInt(extensions.getValue(MAX_ACTIVE, "10"));
            long maxWait = Long.parseLong(extensions.getValue(MAX_WAIT, "10000"));
            int maxIdle = Integer.parseInt(extensions.getValue(MAX_IDLE, "5"));
            long timeBetweenEvictionTimeMillis = Long.parseLong(extensions.getValue(
                    TIME_BETWEEN_EVICTION_RUNS_MILLIS, "10000"));
            long minEvictableIdleTimeMillis = Long.parseLong(extensions.getValue(
                    MIN_EVICTABLE_IDLE_TIME_MILLIS, "2000"));
            boolean testWhileIdle = Boolean.parseBoolean(extensions.getValue(TEST_WHILE_IDLE,
                    "true"));

            component.getJmsProducerConnections().put(
                    provides,
                    new GenericObjectPool(new JMSProviderConnectionFactory(extensions, suRootPath,
                            component.getLogger()), maxActive,
                            GenericObjectPool.WHEN_EXHAUSTED_BLOCK, maxWait, maxIdle, true, false,
                            timeBetweenEvictionTimeMillis, 5, minEvictableIdleTimeMillis,
                            testWhileIdle));
        } else {
            component.getJmsProducerConnections().put(
                    provides,
                    new GenericObjectPool(new JMSProviderConnectionFactory(extensions, suRootPath,
                            component.getLogger()), 10, GenericObjectPool.WHEN_EXHAUSTED_BLOCK,
                            10000l, 5, true, false, 10000l, 5, 2000l, true));
        }
    }

    private void unregisterConnectionPool(Provides provides) throws TechnicalException {
        // close pool
        GenericObjectPool pool = component.getJmsProducerConnections().get(provides);
        try {
            pool.close();
        } catch (Exception e) {
            throw new TechnicalException("Can't close pool associated to provides node : "
                    + provides);
        }
        // remove pool
        component.getJmsProducerConnections().remove(provides);
    }

}
